@@ -3,7 +3,7 @@ from .lib import SUBHALO_DTYPE, HISTORY_DTYPE, BRANCH_DTYPE, CORE_DTYPE
 # I/O function
 from .lib import read_subhalos, read_cores, read_branches, read_tree, read_particles, ParticleInfo
 # Utilities
-from .lib import simulation_parameters, parameter_table, scale_factors, pristine_merger_indices, merger_stats, propagate_parent_indices, colossus_parameters, suite_names, merger_lookup_table, find_merger_branch, find_all_merger_branches
+from .lib import simulation_parameters, parameter_table, scale_factors, pristine_merger_indices, merger_stats, propagate_parent_indices, colossus_parameters, suite_names, merger_lookup_table, find_merger_branch, find_all_merger_branches, is_real_confirmed
 # TODO: better names for pristine_merger_indices and propagate_parent_indices
 
 # Abstract models
@@ -11,9 +11,11 @@ from .star_tagging import ProfileModel, RHalfModel, MStarModel, AbstractRanking
 # Profile models
 from .star_tagging import PlummerProfile
 # R_half models
-from .star_tagging import Nadler2020RHalf, Jiang2019RHalf
+from .star_tagging import Nadler2020RHalf, Jiang2019RHalf, FixedRHalf
+# Metallicity models
+from .star_tagging import Kirby2013Metallicity
 # M_star models
-from .star_tagging import UniverseMachineMStar
+from .star_tagging import UniverseMachineMStarFit
 # Ranking schemes
 from .star_tagging import RadialEnergyRanking
 # Galaxy-halo models
@@ -28,3 +30,6 @@ from .util import  get_host_directory, n_hosts
 
 # These are functions that are only really useful for the the tutorial
 from .util import plot_circle
+
+# File management
+from .file_management import pack_files, download_packed_files, download_files, unpack_files
