@@ -41,7 +41,7 @@ HISTORY_DTYPE = [("mpeak", "f4"), ("vpeak", "f4"), ("merger_snap", "i4"),
                  ("start", "i4"), ("end", "i4"), ("is_real", "?"),
                  ("is_disappear", "?"), ("is_main_sub", "?"),
                  ("preprocess", "i4"), ("first_infall_snap", "i4"),
-                 ("branch_idx", "i4")]
+                 ("branch_idx", "i4"), ("false_selection", "?")]
 
 
 """ BRANCH_DTYPE is a numpy datatype representing the main branch of of halo's
@@ -133,6 +133,8 @@ the number of snapshots in the suite.
 parameter_table = {
     "SymphonyLMC": _chinchilla_cosmology.copy(),
     "SymphonyMilkyWay": _chinchilla_cosmology.copy(),
+    "SymphonyMilkyWayLR": _chinchilla_cosmology.copy(),
+    "SymphonyMilkyWayHR": _chinchilla_cosmology.copy(),
     "SymphonyGroup": _chinchilla_cosmology.copy(),
     "SymphonyLCluster": _banerjee_cosmology.copy(),
     "SymphonyCluster": _carmen_cosmology.copy(),
@@ -141,6 +143,8 @@ parameter_table = {
 
 parameter_table["SymphonyLMC"]["eps"] = 0.080
 parameter_table["SymphonyMilkyWay"]["eps"] = 0.170
+parameter_table["SymphonyMilkyWayLR"]["eps"] = 0.170
+parameter_table["SymphonyMilkyWayHR"]["eps"] = 0.170/2
 parameter_table["SymphonyGroup"]["eps"] = 0.360
 parameter_table["SymphonyLCluster"]["eps"] = 1.200
 parameter_table["SymphonyCluster"]["eps"] = 3.250
@@ -148,6 +152,8 @@ parameter_table["MWest"]["eps"] = 0.170
 
 parameter_table["SymphonyLMC"]["mp"] = 3.52476e4
 parameter_table["SymphonyMilkyWay"]["mp"] = 2.81981e5
+parameter_table["SymphonyMilkyWayLR"]["mp"] = 2.81981e5
+parameter_table["SymphonyMilkyWayHR"]["mp"] = 2.81981e5/8
 parameter_table["SymphonyGroup"]["mp"] = 2.25585e6
 parameter_table["SymphonyLCluster"]["mp"] = 1.51441632e8
 parameter_table["SymphonyCluster"]["mp"] = 1.26201360e8
@@ -155,6 +161,8 @@ parameter_table["MWest"]["mp"] = 2.81981e5
 
 parameter_table["SymphonyLMC"]["n_snap"] = 236
 parameter_table["SymphonyMilkyWay"]["n_snap"] = 236
+parameter_table["SymphonyMilkyWayLR"]["n_snap"] = 236
+parameter_table["SymphonyMilkyWayHR"]["n_snap"] = 236
 parameter_table["SymphonyGroup"]["n_snap"] = 236
 parameter_table["SymphonyLCluster"]["n_snap"] = 200
 parameter_table["SymphonyCluster"]["n_snap"] = 200
@@ -169,8 +177,7 @@ parameter_table["ExampleSuite"] = parameter_table["MWest"]
 def suite_names():
     """ suite_names returns the names of all the simulation suites.
     """
-    return ["SymphonyLMC", "SymphonyMilkyWay", "SymphonyGroup",
-            "SymphonyLCluster", "SymphonyCluster", "MWest"]
+    return util.SUITE_NAMES
 
 def simulation_parameters(suite_name):
     """ parameter_table contains simulation-defining parameters. This includes a
@@ -206,21 +213,51 @@ def halo_dir_to_suite_name(dir_name):
     base_dir, suite_name = os.path.split(suite_dir)
     return suite_name
 
+_SCALE_FACTOR_CACHE = { }
+
 def scale_factors(dir_name):
     """ scale_factors returns the scale factors of each snapshot for a halo in
     the given directory.
     """
+    if dir_name in _SCALE_FACTOR_CACHE:
+        return _SCALE_FACTOR_CACHE[dir_name]
+
     # TODO: individual halo-by-halo scale factors
     suite_name = halo_dir_to_suite_name(dir_name)
-    if suite_name in ["SymphonyLMC", "SymphonyGroup",
-                      "SymphonyMilkyWay", "MWest"]:
-        return 10**np.linspace(np.log10(0.05), np.log10(1), 236)
-    elif suite_name in ["SymphonyLCluster",  "SymphonyCluster"]:
-        return 10**np.linspace(np.log10(0.075), np.log10(1), 200)
+    if (suite_name in ["SymphonyLMC", "SymphonyGroup",
+                      "SymphonyMilkyWay", "MWest", "SymphonyMilkyWayLR",
+                       "SymphonyMilkyWayHR"] or
+        dir_name in ["SymphonyLMC", "SymphonyGroup",
+                     "SymphonyMilkyWay", "MWest", "SymphonyMilkyWayLR",
+                     "SymphonyMilkyWayHR"]):
+        default = 10**np.linspace(np.log10(0.05), np.log10(1), 236)
+    elif (suite_name in ["SymphonyLCluster",  "SymphonyCluster"] or
+          dir_name in ["SymphonyLCluster",  "SymphonyCluster"]):
+        default = 10**np.linspace(np.log10(0.075), np.log10(1), 200)
     else:
-        raise ValueError(("The halo in %s does not bleong to a " + 
+        raise ValueError(("The halo in %s does not belong to a " + 
                           "recognized suite.") % dir_name)
 
+    if dir_name in ["SymphonyLMC", "SymphonyGroup",
+                    "SymphonyMilkyWay", "MWest", "SymphonyMilkyWayLR",
+                    "SymphonyMilkyWayHR", 
+                    "SymphonyLCluster",  "SymphonyCluster"]:
+        return default
+
+    file_name = path.join(dir_name, "halos", "snap_table.dat")
+    if not path.exists(file_name):
+        _SCALE_FACTOR_CACHE[dir_name] = default
+        return default
+
+    f = open(fname, "rb")
+    n_snap = struct.unpack("q", f.read(8))[0]
+    scale = np.fromfile(f, np.float64, n_snap)
+    f.close()
+
+    _SCALE_FACTOR_CACHE[dir_name] = scale
+    return scale
+
+
 def mvir_to_rvir(mvir, a, omega_M):
     """ mvir_to_rvir converts a Bryan & Norman virial mass in Msun/h to a virial
     radius in comoving Mpc/h at a given scale factor a, and omega_M.
@@ -278,7 +315,7 @@ def merger_snap(h, x_sub, snap_sub):
     dist = np.sqrt(np.sum((h["x"][snap_sub] - x_sub)**2, axis=1))
     within = dist < h["rvir"][snap_sub]
     merger = h["ok"][snap_sub] & within
-    
+
     if np.sum(merger) == 0:
         return -1
     else:
@@ -315,7 +352,7 @@ def merger_stats(b, m, x, mvir, snap):
 
     return mpeak, m_snap, ratio, sub_idx
 
-def read_subhalos(dir_name, comoving=False):
+def read_subhalos(dir_name, comoving=False, include_false_selections=False):
     """ read_subhalos reads major merger data from the halo directory dir_name.
     It returns two arrays. The first, m_idx, is the indices of the major
     mergers within the branches arrays. Index 0 is the main halo, index 1 is the
@@ -325,6 +362,11 @@ def read_subhalos(dir_name, comoving=False):
     halo at the index halo_idx in m_idx and the snapshot snap. The fields are
     given by SUBHALO_DTYPE (see the header of this file). If a halo doesn't
     exist at a given snapshot, values are set -1 ok will be set to false.
+
+    If comoving is True, the subhaloes will be returned in Rockstar's default,
+    comoving units. Otherwise it's returned in symlib's units. If
+    include_false_slections is true, haloes which would be included with a
+    straight M_peak > 300*mp cutoff are included. If False (the default)
     """
     params = simulation_parameters(dir_name)
 
@@ -334,7 +376,7 @@ def read_subhalos(dir_name, comoving=False):
     n_snap = struct.unpack("i", f.read(4))[0]
     n_merger = struct.unpack("i", f.read(4))[0]
 
-    idx = np.fromfile(f, np.int32, n_merger)    
+    idx = np.fromfile(f, np.int32, n_merger)
     out = np.zeros((n_merger, n_snap), dtype=SUBHALO_DTYPE)
     a = scale_factors(dir_name)
 
@@ -378,6 +420,11 @@ def read_subhalos(dir_name, comoving=False):
         scale = scale_factors(dir_name)
         out = util.set_units_halos(out, scale, params)
         histories = util.set_units_histories(histories, scale, params)
+
+    if not include_false_selections:
+        out = out[~histories["false_selection"]]
+        histories = histories[~histories["false_selection"]]
+
     return out, histories
 
 def get_subhalo_histories(s, idx, dir_name):
@@ -385,6 +432,7 @@ def get_subhalo_histories(s, idx, dir_name):
         raise ValueError("The input subhalo array has already been " + 
                          "converted to symlib's default units.")
 
+    param = simulation_parameters(dir_name)
     b = read_branches(dir_name)
     h = np.zeros(len(s), dtype=HISTORY_DTYPE)
 
@@ -411,11 +459,19 @@ def get_subhalo_histories(s, idx, dir_name):
         m_snap = merger_snap(central, x[ok], snap[ok])
         m_ratio = mvir[m_snap] / central["mvir"][m_snap]
 
+        target = (snap >= m_snap) & s["ok"][i]
+
         h[i]["merger_snap"], h[i]["merger_ratio"] = m_snap, m_ratio
-        
+        h["first_infall_snap"][i] = min(h["merger_snap"][i],
+                                        h["first_infall_snap"][i])
+
+        infall_snap = h["first_infall_snap"][i]
+        mpeak_infall = np.max(s[i,:infall_snap+1]["mvir"])
+        h["false_selection"][i] = mpeak_infall < 300*param["mp"]
+
     return h
 
-def read_cores(dir_name):
+def read_cores(dir_name, include_false_selections=False):
     """ read_cores read the particle-tracked halo cores of the halo in the
     given directory. The returned array is a structured array of type CORE_DTYPE
     with shape (n_halos, n_snaps).
@@ -441,7 +497,13 @@ def read_cores(dir_name):
 
         out["ok"] = out["m_bound"] != -1
 
-    return out.reshape((n_halo, n_snap))
+    out = out.reshape((n_halo, n_snap))
+
+    if not include_false_selections:
+        h, hist = read_subhalos(dir_name, include_false_selections=True)
+        out = out[~hist["false_selection"]]
+
+    return out
 
 def read_branches(dir_name):
     """ read_branches reads main branch data from the halo directory dir_name.
@@ -623,7 +685,8 @@ class ParticleTags(object):
         self.flag = [None]*part_hd.n_halo
 
         for i_file in range(part_hd.n_file):
-            file_name = path.join(base_dir, "particles", "tags.%d.dat" % i_file)
+            file_name = path.join(base_dir, "particles",
+                                  "tags.%d.dat" % i_file)
             f = open(file_name, "rb")
             
             to_read = np.where(i_file == part_hd.file_idxs)[0]
@@ -635,6 +698,32 @@ class ParticleTags(object):
             f.close()
 
         self.n0 = part_hd.n0
+        self.n_file = part_hd.n_file
+        self.n_halo =  part_hd.n_halo
+        self.file_idxs = part_hd.file_idxs
+
+        self._n0_snaps = [self.snap[i][self.flag[i] == 0]
+                          for i in range(part_hd.n_halo)]
+        self._offset_lookup = {}
+
+    def halo_offset(self, snap, i_halo):
+        if snap in self._offset_lookup:
+            return self._offset_lookup[snap][i_halo]
+        else:
+            offsets = np.zeros(self.n_halo, dtype=int)
+            for i_file in range(self.n_file):
+                to_read = np.where(i_file == self.file_idxs)[0]
+                curr_offset = 4
+                for i in to_read:
+                    offsets[i] = curr_offset
+
+                    if len(self._n0_snaps[i]) == 0: continue
+                    n = np.sum(self._n0_snaps[i] <= snap)
+                    if n == 0: continue
+
+                    curr_offset += 6*n + 8 + 12 + 12
+            self._offset_lookup[snap] = offsets
+            return self._offset_lookup[snap][i_halo]
 
 class TagLookup(object):
     def __init__(self, base_dir):
@@ -655,25 +744,80 @@ class ParticleInfo(object):
         self.global_offset = global_offset
         self.global_index = global_offset[self.lookup.halo] + self.lookup.index
         
-def read_particles(part_info, base_dir, snap, var_name):
+        # Used to find false selections
+        _, self.hist_false = read_subhalos(
+            base_dir, include_false_selections=True)
+
+def is_real_confirmed(part_info, h, i_sub):
+    first_snap = np.where(h[i_sub]["ok"])[0][0]
+    ok = read_particles(part_info, None, first_snap, "valid", owner=i_sub)
+    return np.sum(ok) > 0
+
+def read_particles(part_info, base_dir, snap, var_name,
+                   owner=None, include_false_selections=False):
     hd, tags = part_info.part_hd, part_info.tags
     
+    h_idx = np.arange(len(part_info.hist_false), dtype=int)
+    if include_false_selections:
+        false_remapping = h_idx
+    else:
+        false_remapping = h_idx[~part_info.hist_false["false_selection"]]
+    if owner is not None and not include_false_selections:
+        owner = false_remapping[owner]
+
     if var_name == "id":
-        return tags.id
+        if owner is None:
+            return [tags.id[i] for i in false_remapping]
+        else:
+            return tags.id[owner][tags.flag[owner] == 0]
     elif var_name == "snap":
-        return tags.snap
+        if owner is None:
+            return [tags.snap[i] for i in false_remapping]
+        else:
+            return tags.snap[owner][tags.flag[owner] == 0]
     elif var_name == "ownership":
-        return tags.flag
+        if owner is None:
+            return [tags.flag[i] for i in false_remapping]
+        else:
+            return tags.flag[owner][tags.flag[owner] == 0]
     elif var_name == "valid":
-        valid = [None]*len(tags.snap)
-        for i in range(len(valid)):
-            valid[i] = tags.snap[i] <= snap
-        return valid
+        if owner is None:
+            valid = [None]*len(tags.snap)
+            for i in range(len(valid)):
+                valid[i] = tags.snap[i] <= snap
+            return [valid[i] for i in false_remapping]
+        else:
+            valid = (tags.snap[owner] <= snap)[tags.flag[owner] == 0]
+            return valid
     elif var_name in ["x", "v"]:
         x_full = np.zeros((hd.n_particle, 3))
 
+        snap_name = "snap_%03d" % snap
+        if owner is not None:
+            i_file = hd.file_idxs[owner]
+            file_name = "%s.%d.dat" % (var_name, i_file)
+
+            path = os.path.join(hd.base_dir, "particles", snap_name, file_name)
+            f = open(path, "rb")
+
+            code = struct.unpack("i", f.read(4))[0]
+            if code != 0: raise ValueError("%s is not a vector file." % path)
+
+            offset = tags.halo_offset(snap, owner)
+            f.seek(offset, 0)
+
+            size = struct.unpack("q", f.read(8))[0]
+            min = np.array(struct.unpack("fff", f.read(12)))
+            max = np.array(struct.unpack("fff", f.read(12)))
+            
+            qx_i = np.fromfile(f, np.uint16, size*3)
+            x_i = _dequantize_vector(qx_i, min, max)
+            x_i = _expand_vector(tags, x_i, owner, snap)
+
+            f.close()
+            return x_i
+
         for i_file in range(hd.n_file):
-            snap_name = "snap_%03d" % snap
             file_name = "%s.%d.dat" % (var_name, i_file)
             path = os.path.join(hd.base_dir, "particles", snap_name, file_name)
             f = open(path, "rb")
@@ -687,6 +831,8 @@ def read_particles(part_info, base_dir, snap, var_name):
                 ok = tags.snap[i_halo][tags.flag[i_halo] == 0] <= snap
                 if np.sum(ok) == 0: continue
 
+                curr_offset = f.seek(0, 1)
+
                 size = struct.unpack("q", f.read(8))[0]
                 min = np.array(struct.unpack("fff", f.read(12)))
                 max = np.array(struct.unpack("fff", f.read(12)))
@@ -705,23 +851,17 @@ def read_particles(part_info, base_dir, snap, var_name):
             idx = part_info.global_index[tags.id[i_halo] - 1]
             out[i_halo] = x_full[idx]
 
-        return out
+        return [out[i] for i in false_remapping]
     elif var_name == "infall_core":
+        # Setting owner doesn't do anything for infall_core.
+
         file_name = os.path.join(base_dir, "halos", "infall_cores.dat")
     
         with open(file_name, "rb") as fp:
             n_halo, n_core = struct.unpack("qq", fp.read(16))
             idxs = np.fromfile(fp, dtype=np.int32, count=n_halo*n_core)
             idxs = idxs.reshape((n_halo, n_core))
-        return idxs
-    #elif var_name == "core":
-    #    file_name = os.path.join(base_dir, "halos", "cores.dat")
-    #
-    #    with open(file_name, "rb") as fp:
-    #        n_halo, n_snap, n_core = struct.unpack("qqq", fp.read(24))
-    #        idxs = np.fromfile(fp, dtype=np.int32, count=n_halo*n_snap*n_core)
-    #        idxs = idxs.reshape(n_halo, n_snap, n_core)
-    #    return idxs
+        return [idxs[i] for i in false_remapping]
     else:
         raise ValueError("Unknown property name, '%s'" % var_name)    
     
