@@ -70,6 +70,10 @@ def set_units_halos(h, scale, param):
         x0 = np.copy(h["x"][0,:,dim])
         v0 = np.copy(h["v"][0,:,dim])
 
+        if x0[-1] == 0:
+            raise ValueError("Calling set_units_halos() on a set of halos " +
+                             "already in physical units.")
+
         for hi in range(len(h)):
             h["x"][hi,:,dim] -= x0
             h["v"][hi,:,dim] -= v0
@@ -104,6 +108,15 @@ DEFAULT_HALO_NAMES = {
         "Halo188", "Halo327", "Halo415", "Halo490", "Halo606", "Halo738", "Halo829", "Halo926", "Halo9829",
         "Halo247", "Halo349", "Halo416", "Halo530", "Halo628", "Halo749", "Halo852", "Halo937", "Halo990"
     ]),
+
+    "SymphonyMilkyWayLR": sorted([
+        "Halo023", "Halo247", "Halo268", "Halo530", "Halo829"
+    ]),
+
+    "SymphonyMilkyWayHR": sorted([
+        "Halo023", "Halo247", "Halo268", "Halo530", "Halo829"
+    ]),    
+
     "SymphonyGroup": sorted([
         "Halo015", "Halo175", "Halo313", "Halo352", "Halo470", "Halo579", "Halo755", "Halo806", "Halo985",
         "Halo024", "Halo183", "Halo331", "Halo383", "Halo496", "Halo581", "Halo759", "Halo834",
@@ -130,7 +143,7 @@ DEFAULT_HALO_NAMES = {
         "Halo266", "Halo324", "Halo358", "Halo391", "Halo437", "Halo465", "Halo494", "Halo574",
         "Halo274", "Halo326", "Halo360", "Halo394", "Halo441", "Halo471", "Halo502", "Halo595",
         "Halo277", "Halo335", "Halo361", "Halo400", "Halo445", "Halo472", "Halo517", "Halo600",
-        "Halo282", "Halo337", "Halo366", "Halo407", "Halo447", "Halo474", "Halo518", "Halo604"
+        "Halo282", "Halo337", "Halo366", "Halo407", "Halo447", "Halo474", "Halo518", "Halo604", "Halo372", "Halo425"
     ]),
     "MWest": sorted([
         "Halo004", "Halo170", "Halo282", "Halo407", "Halo523", "Halo666", "Halo756", "Halo908", "Halo983",
@@ -139,7 +152,7 @@ DEFAULT_HALO_NAMES = {
     ]),
 }
 
-SUITE_NAMES = ["SymphonyLMC", "SymphonyMilkyWay", "SymphonyGroup", "SymphonyLCluster", "SymphonyCluster", "MWest"]
+SUITE_NAMES = ["SymphonyLMC", "SymphonyMilkyWay", "SymphonyMilkyWayLR", "SymphonyMilkyWayHR", "SymphonyGroup", "SymphonyLCluster", "SymphonyCluster", "MWest"]
 
 def n_hosts(suite_name):
     """ n_halos returns the number of host halos in a given suite,
@@ -158,7 +171,12 @@ def get_host_directory(base_dir, suite_name, halo_name):
     halos in a suite, you'll want to use this second option.
     """
     
+    if len(suite_name) > 2 and suite_name[-2:] == "LR":
+        suite_dir = suite_name[:-2]
+    else:
+        suite_dir = suite_name
+
     if isinstance(halo_name, int):
         halo_name = DEFAULT_HALO_NAMES[suite_name][halo_name]
 
-    return path.join(base_dir, suite_name, halo_name)
+    return path.join(base_dir, suite_dir, halo_name)
